﻿using Infragistics.Controls.Editors;
using Infragistics.Controls.Menus;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace ComboEditorTreeDropdown
{
    public class MyComboEditor : XamComboEditor
    {
        #region NodeLayouts

        public NodeLayoutCollection NodeLayouts
        {
            get { return (NodeLayoutCollection)GetValue(NodeLayoutsProperty); }
            set { SetValue(NodeLayoutsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for NodeLayouts.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty NodeLayoutsProperty =
            DependencyProperty.Register("NodeLayouts", typeof(NodeLayoutCollection), typeof(MyComboEditor), new PropertyMetadata(new NodeLayoutCollection(), OnNodeLayoutsChanged));

        private static void OnNodeLayoutsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var combo = d as MyComboEditor;

            if (e.OldValue != null)
            {
                var nodeLayouts = e.OldValue as NodeLayoutCollection;
                nodeLayouts.CollectionChanged -= combo.OnNodeLayoutCollectionChanged;
            }
            if (e.NewValue != null)
            {
                var nodeLayouts = e.NewValue as NodeLayoutCollection;
                nodeLayouts.CollectionChanged += combo.OnNodeLayoutCollectionChanged;
                combo.VerifyNodeLayoutState();
            }
        }

        #endregion

        #region SelectedTreeItem

        public object SelectedTreeItem
        {
            get { return (object)GetValue(SelectedTreeItemProperty); }
            set { SetValue(SelectedTreeItemProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedTreeItem.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SelectedTreeItemProperty =
            DependencyProperty.Register("SelectedTreeItem", typeof(object), typeof(MyComboEditor), new PropertyMetadata(null));

        #endregion

        #region SelectedTreeItems

        public object[] SelectedTreeItems
        {
            get { return (object[])GetValue(SelectedTreeItemsProperty); }
            set { SetValue(SelectedTreeItemsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedTreeItems.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SelectedTreeItemsProperty =
            DependencyProperty.Register("SelectedTreeItems", typeof(object[]), typeof(MyComboEditor), new PropertyMetadata(null));

        #endregion

        #region NodeSelection

        public TreeSelectionType NodeSelection
        {
            get { return (TreeSelectionType)GetValue(NodeSelectionProperty); }
            set { SetValue(NodeSelectionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for NodeSelection.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty NodeSelectionProperty =
            DependencyProperty.Register("NodeSelection", typeof(TreeSelectionType), typeof(MyComboEditor), new PropertyMetadata(TreeSelectionType.Single, OnNodeSelectionChanged));

        private static void OnNodeSelectionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var combo = d as MyComboEditor;
            if (combo._tree != null)
                combo._tree.SelectionSettings.NodeSelection = combo.NodeSelection;
        }

        #endregion

        private XamDataTree _tree;
        private Border _rootPopupElement;
        private Panel _itemsPanel;

        public MyComboEditor()
        {
            this.DefaultStyleKey = typeof(MyComboEditor);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            _tree = (XamDataTree)this.Template.FindName("dataTree1", this);

            _rootPopupElement = (Border)this.Template.FindName("RootPopupElement", this);

            _itemsPanel = (Panel)this.Template.FindName("ItemsPanel", this);

            _tree.SelectionSettings.NodeSelection = this.NodeSelection;

            // bind the SelectedTreeItems property to the XamDataTrees SelectedDataItems property.  This can't be done in XAML due to a .NET limitation when binding to
            // arrays: https://connect.microsoft.com/VisualStudio/feedback/details/1029028/error-when-trying-to-set-a-dependencyproperty-whose-propertytype-is-an-array-within-a-datatemplate
            Binding b = new Binding("SelectedTreeItems");
            b.RelativeSource = new RelativeSource(RelativeSourceMode.TemplatedParent);
            b.Mode = BindingMode.TwoWay;
            BindingOperations.SetBinding(_tree, XamDataTree.SelectedDataItemsProperty, b);

            VerifyNodeLayoutState();
        }

        void OnNodeLayoutCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // if node layouts are added or removed or the entire collection is reset then we need to update the internal XamDataTree
            VerifyNodeLayoutState();
        }

        private void VerifyNodeLayoutState()
        {
            if (this.NodeLayouts != null)
            {
                _tree.GlobalNodeLayouts.Clear();
                foreach (var nodeLayout in this.NodeLayouts)
                    _tree.GlobalNodeLayouts.Add(nodeLayout);
            }
        }
    }
}
